@@ -10,16 +10,14 @@
 /* xx TODO add R_CheckUserInterrupt(); */
 
 #include "r_ff.h"
-
+#include "r_file_resize.h"
 /* { --- R FUNCTION REGISTRATION -------------------------------------------- */
 
 R_CallMethodDef callMethods[] =
 {
  {"ffxtensions", (DL_FUNC) &r_ff_xtensions_available, 0},
+ {"ffsymmxtensions", (DL_FUNC) &r_ff_symmxtensions_available, 0},
  {"getpagesize", (DL_FUNC) &r_ff_getpagesize, 0},
- {"int_check_ascending", (DL_FUNC) &r_ff_int_check_ascending, 1},
- {"int_check_descending", (DL_FUNC) &r_ff_int_check_descending, 1},
- {"int_rle", (DL_FUNC) &r_ff_int_rle, 1},
  {"geterror", (DL_FUNC) &r_ff_geterror, 1},
  {"geterrstr", (DL_FUNC) &r_ff_geterrstr, 1},
  {"ffmode_implemented",  (DL_FUNC) &r_ff_ffmode_implemented,   1},
@@ -54,7 +52,7 @@ R_CallMethodDef callMethods[] =
  {"addgetset_array", (DL_FUNC) &r_ff__addgetset_array, 10},
  {"addset_array",    (DL_FUNC) &r_ff__addset_array,    10},
 
-#if FF_XTENSIONS_AVAILABLE
+#if FF_SYMMEXTENSIONS_AVAILABLE
  {"getset_symm",   (DL_FUNC) &r_ff__getset_symm, 7},
  {"get_symm",      (DL_FUNC) &r_ff__get_symm,    6},
  {"set_symm",      (DL_FUNC) &r_ff__set_symm,    7},
@@ -65,6 +63,9 @@ R_CallMethodDef callMethods[] =
  {"subset_symm", (DL_FUNC) &r_ram__set_symm, 8},
 #endif
 
+/* additions by Daniel Adler (under ISC license) */
+ {"r_file_resize", (DL_FUNC) &r_file_resize, 2 },
+
  {NULL, NULL, 0}
 };
 
@@ -264,6 +265,15 @@ SEXP r_ff_xtensions_available()
   return ret_;
 }
 
+SEXP r_ff_symmxtensions_available()
+{
+  SEXP ret_;
+  PROTECT( ret_ = allocVector(LGLSXP,1) );
+  LOGICAL(ret_)[0] = FF_SYMMXTENSIONS_AVAILABLE;
+  UNPROTECT(1);
+  return ret_;
+}
+
 
 
 SEXP r_ff_getpagesize()
@@ -275,139 +285,6 @@ SEXP r_ff_getpagesize()
   return ret_;
 }
 
-/* check for NA and not sorted ascending
-   stop on finding NA but not on finding unsorted
-   (guarantee any NA is found)
-   return 0 (OK) 1 (unsorted) 2 (has NA)
-*/
-SEXP r_ff_int_check_ascending(SEXP x)
-{
-  Rboolean status=TRUE;
-  int i;
-  int n = LENGTH(x);
-  int *p = INTEGER(x);
-  SEXP ret_;
-  PROTECT( ret_ = allocVector(LGLSXP, 1) );
-
-  if (n){
-    if (p[0]==NA_INTEGER){
-      status=NA_LOGICAL;
-    }else{
-      for (i=1;i<n;i++){
-        if (p[i]==NA_INTEGER){
-          status = NA_LOGICAL;
-          break;
-        }else if (p[i]<p[i-1]){
-          status = FALSE;
-        }
-      }
-    }
-  }
-
-  INTEGER(ret_)[0] = status;
-  UNPROTECT(1);
-  return ret_;
-}
-
-/* check for not sorted descending
-   stop on finding unsorted
-   (assume check for NAs has already be done successfully: no NAs)
-   return 0 (OK) 1 (unsorted)
-*/
-SEXP r_ff_int_check_descending(SEXP x)
-{
-  Rboolean status=TRUE;
-  int i;
-  int n = LENGTH(x);
-  int *p = INTEGER(x);
-  SEXP ret_;
-  PROTECT( ret_ = allocVector(LGLSXP, 1) );
-
-  if (n){
-    for (i=1;i<n;i++){
-      if (p[i]>p[i-1]){
-        status = FALSE;
-        break;
-      }
-    }
-  }
-
-  INTEGER(ret_)[0] = status;
-  UNPROTECT(1);
-  return ret_;
-}
-
-
-
-/* create integer rle
-   NOTE if rle is not efficient we return NULL instead of an rle object
-*/
-SEXP r_ff_int_rle(SEXP x_)
-{
-  register int lv,ln,i,c=0;
-  int n2, n = LENGTH(x_);
-  if (n<3)
-    return R_NilValue;
-  n2 = n / 3; /* xx max RAM requirement 2x, but rle only if at least 2/3 savings, using 2 instead of 3 would need 50% more time, have max RAM requirement 2.5x for savings of any size */
-
-  int *x = INTEGER(x_);
-  int *val, *len, *values, *lengths;
-  SEXP ret_, lengths_, values_, names_, class_;
-
-  val = Calloc(n2, int);
-  len = Calloc(n2, int);
-  if (n){
-    lv = x[0];
-    ln = 1;
-    for (i=1;i<n;i++){
-      if (x[i]==lv){
-        ln++;
-      }else{
-        val[c] = lv;
-        len[c] = ln;
-        c++;
-        if (c==n2){
-          Free(val);
-          Free(len);
-          return R_NilValue;
-        }
-        lv = x[i];
-        ln = 1;
-      }
-    }
-    val[c] = lv;
-    len[c] = ln;
-    c++;
-  }
-  PROTECT( values_ = allocVector(INTSXP, c) );
-  values = INTEGER(values_);
-  for (i=0;i<c;i++)
-    values[i] = val[i];
-  Free(val);
-  PROTECT( lengths_ = allocVector(INTSXP, c) );
-  lengths = INTEGER(lengths_);
-  for (i=0;i<c;i++)
-    lengths[i] = len[i];
-  Free(len);
-
-  PROTECT( ret_ = allocVector(VECSXP, 2) );
-  PROTECT( names_ = allocVector(STRSXP, 2));
-  PROTECT( class_ = allocVector(STRSXP, 1));
-
-  SET_STRING_ELT(names_, 0, mkChar("lengths"));
-  SET_STRING_ELT(names_, 1, mkChar("values"));
-  SET_STRING_ELT(class_, 0, mkChar("rle"));
-  SET_VECTOR_ELT(ret_, 0, lengths_);
-  SET_VECTOR_ELT(ret_, 1, values_);
-  setAttrib(ret_, R_NamesSymbol, names_);
-  classgets(ret_, class_);
-
-  UNPROTECT(5);
-  return ret_;
-}
-
-
-
 
 
 /* } --- UTILITIES ---------------------------------------------------------- */
@@ -796,7 +673,7 @@ FUNA(1,D)
 
 /* make the switch functions that switch methods conditional on vmode */
 #include "r_ff_methodswitch.h"
-#if FF_XTENSIONS_AVAILABLE
+#if FF_SYMMXTENSIONS_AVAILABLE
   #include "x_r_ff_methodswitch.h"
 #endif
 
